@@ -4,82 +4,82 @@
 <context>
     <name>A11yKeyboardManager</name>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="479"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="491"/>
         <source>There was an error displaying help</source>
         <translation>显示帮助时出错</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="641"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="653"/>
         <source>Do you want to activate Slow Keys?</source>
         <translation>您想激活慢速键吗？</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="642"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="654"/>
         <source>Do you want to deactivate Slow Keys?</source>
         <translation>您想要停用慢速键吗？</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="643"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="655"/>
         <source>You just held down the Shift key for 8 seconds.  This is the shortcut for the Slow Keys feature, which affects the way your keyboard works.</source>
         <translation>您只是按下 Shift 键8秒。这是慢速键功能的快捷方式，它会影响键盘的工作方式。</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="652"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="664"/>
         <source>Slow Keys Alert</source>
         <translation>慢速键警告</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="656"/>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="788"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="668"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="800"/>
         <source>Do_n&apos;t activate</source>
         <translation>不要激活</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="656"/>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="788"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="668"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="800"/>
         <source>Do_n&apos;t deactivate</source>
         <translation>不要关闭</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="658"/>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="790"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="670"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="802"/>
         <source>_Activate</source>
         <translation>激活</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="658"/>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="790"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="670"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="802"/>
         <source>_Deactivate</source>
         <translation>关闭</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="660"/>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="791"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="672"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="803"/>
         <source>input-keyboard</source>
         <translation>输入键盘</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="770"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="782"/>
         <source>Do you want to activate Sticky Keys?</source>
         <translation>您想激活粘滞键吗？</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="771"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="783"/>
         <source>Do you want to deactivate Sticky Keys?</source>
         <translation>您想关闭粘滞键吗？</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="773"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="785"/>
         <source>You just pressed the Shift key 5 times in a row.  This is the shortcut for the Sticky Keys feature, which affects the way your keyboard works.</source>
         <translation>您只是连续按了5次 Shift 键。这是慢速键特性的快捷方式，它会影响键盘的工作方式。</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="775"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="787"/>
         <source>You just pressed two keys at once, or pressed the Shift key 5 times in a row.  This turns off the Sticky Keys feature, which affects the way your keyboard works.</source>
         <translation>您只需同时按两个键，或者连续按5次 Shift 键。这将关闭影响键盘工作方式的粘滞键功能。</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="784"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="796"/>
         <source>Sticky Keys Alert</source>
         <translation>粘滞键警报</translation>
     </message>
@@ -146,6 +146,14 @@
         <translation>设备窗口</translation>
     </message>
 </context>
+<context>
+    <name>KeyboardWidget</name>
+    <message>
+        <location filename="../../../plugins/keyboard/keyboardwidget.ui" line="16"/>
+        <source>Form</source>
+        <translation type="unfinished">形成</translation>
+    </message>
+</context>
 <context>
     <name>LdsmDialog</name>
     <message>
@@ -154,47 +162,62 @@
         <translation>对话框</translation>
     </message>
     <message>
-        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="112"/>
+        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="92"/>
+        <source>Low Disk Space</source>
+        <translation>磁盘空间小</translation>
+    </message>
+    <message>
+        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="120"/>
         <source>Ignore</source>
         <translation>忽视</translation>
     </message>
     <message>
-        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="117"/>
+        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="125"/>
         <source>Empty Trash</source>
         <translation>清空回收站</translation>
     </message>
     <message>
-        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="121"/>
+        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="129"/>
         <source>Examine</source>
         <translation>检查</translation>
     </message>
     <message>
-        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="146"/>
+        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="142"/>
+        <source>The volume &quot;%1&quot; has only %s disk space remaining.</source>
+        <translation>卷 &quot;%1&quot; 仅剩余 %s 硬盘空间。</translation>
+    </message>
+    <message>
+        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="145"/>
+        <source>The computer has only %s disk space remaining.</source>
+        <translation>本计算机仅剩余 %s 硬盘空间。</translation>
+    </message>
+    <message>
+        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="154"/>
         <source>You can free up disk space by emptying the Trash, removing unused programs or files, or moving files to another disk or partition.</source>
         <translation>您可以通过清空垃圾箱，删除未使用的程序或文件或将文件移动到另一个磁盘或分区来释放磁盘空间。</translation>
     </message>
     <message>
-        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="149"/>
+        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="157"/>
         <source>You can free up disk space by removing unused programs or files, or by moving files to another disk or partition.</source>
         <translation>您可以通过删除未使用的程序或文件或将文件移动到另一个磁盘或分区来释放磁盘空间。</translation>
     </message>
     <message>
-        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="153"/>
+        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="161"/>
         <source>You can free up disk space by emptying the Trash, removing unused programs or files, or moving files to an external disk.</source>
         <translation>您可以通过清空垃圾箱，删除未使用的程序或文件或将文件移动到外部磁盘来释放磁盘空间。</translation>
     </message>
     <message>
-        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="156"/>
+        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="164"/>
         <source>You can free up disk space by removing unused programs or files, or by moving files to an external disk.</source>
         <translation>您可以通过删除未使用的程序或文件或将文件移动到外部磁盘来释放磁盘空间。</translation>
     </message>
     <message>
-        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="164"/>
+        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="172"/>
         <source>Don&apos;t show any warnings again for this file system</source>
         <translation>不再为此文件系统显示任何警告</translation>
     </message>
     <message>
-        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="166"/>
+        <location filename="../../../plugins/housekeeping/usd-ldsm-dialog.cpp" line="174"/>
         <source>Don&apos;t show any warnings again</source>
         <translation>不要再显示任何警告</translation>
     </message>
@@ -207,9 +230,29 @@
         <translation>对话框</translation>
     </message>
     <message>
-        <location filename="../../../plugins/housekeeping/ldsm-trash-empty.cpp" line="45"/>
+        <location filename="../../../plugins/housekeeping/ldsm-trash-empty.cpp" line="60"/>
         <source>Emptying the trash</source>
-        <translation>清空垃圾</translation>
+        <translation>清空回收站</translation>
+    </message>
+    <message>
+        <location filename="../../../plugins/housekeeping/ldsm-trash-empty.cpp" line="76"/>
+        <source>Empty all of the items from the trash?</source>
+        <translation>删除回收站里的所有项目？</translation>
+    </message>
+    <message>
+        <location filename="../../../plugins/housekeeping/ldsm-trash-empty.cpp" line="80"/>
+        <source>If you choose to empty the trash, all items in it will be permanently lost.Please note that you can also delete them separately.</source>
+        <translation>如果您选择清空回收站，里面所有的项目都将永久丢失。请注意您也可以分别删除这些项目。</translation>
+    </message>
+    <message>
+        <location filename="../../../plugins/housekeeping/ldsm-trash-empty.cpp" line="83"/>
+        <source>cancel</source>
+        <translation>取消</translation>
+    </message>
+    <message>
+        <location filename="../../../plugins/housekeeping/ldsm-trash-empty.cpp" line="85"/>
+        <source>Empty Trash</source>
+        <translation>清空回收站</translation>
     </message>
 </context>
 <context>
@@ -220,19 +263,19 @@
         <translation type="vanished">试图运行&quot;%1&quot;时出错; 链接到键&quot;%2&quot;</translation>
     </message>
     <message>
-        <location filename="../../../plugins/keybindings/keybindings-manager.cpp" line="398"/>
+        <location filename="../../../plugins/keybindings/keybindings-manager.cpp" line="403"/>
         <source>Error while trying to run &quot;%1&quot;;
  which is linked to the key &quot;%2&quot;</source>
         <translation>尝试运行&quot;%1&quot;时出错；
 链接到快捷键&quot;%2&quot;</translation>
     </message>
     <message>
-        <location filename="../../../plugins/keybindings/keybindings-manager.cpp" line="401"/>
+        <location filename="../../../plugins/keybindings/keybindings-manager.cpp" line="406"/>
         <source>Shortcut message box</source>
         <translation>快捷键消息弹框</translation>
     </message>
     <message>
-        <location filename="../../../plugins/keybindings/keybindings-manager.cpp" line="404"/>
+        <location filename="../../../plugins/keybindings/keybindings-manager.cpp" line="409"/>
         <source>Yes</source>
         <translation>确定</translation>
     </message>
@@ -273,67 +316,59 @@ X服务器版本数据：
         <translation>错误</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="562"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="574"/>
         <source>Do you want to activate Slow Keys?</source>
         <translation>您想激活慢速键吗？</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="563"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="575"/>
         <source>Do you want to deactivate Slow Keys?</source>
         <translation>您想要停用慢速键吗？</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="565"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="577"/>
         <source>You just held down the Shift key for 8 seconds.  This is the shortcut for the Slow Keys feature, which affects the way your keyboard works.</source>
         <translation>您只是按下 Shift 键8秒。这是慢速键功能的快捷方式，它会影响键盘的工作方式。</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="696"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="708"/>
         <source>Do you want to activate Sticky Keys?</source>
         <translation>您想激活粘滞键吗？</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="697"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="709"/>
         <source>Do you want to deactivate Sticky Keys?</source>
         <translation>您想关闭粘滞键吗？</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="699"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="711"/>
         <source>You just pressed the Shift key 5 times in a row.  This is the shortcut for the Sticky Keys feature, which affects the way your keyboard works.</source>
         <translation>您只是连续按了5次 Shift 键。这是粘滞键特性的快捷方式，它会影响键盘的工作方式。</translation>
     </message>
     <message>
-        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="701"/>
+        <location filename="../../../plugins/a11y-keyboard/a11y-keyboard-manager.cpp" line="713"/>
         <source>You just pressed two keys at once, or pressed the Shift key 5 times in a row.  This turns off the Sticky Keys feature, which affects the way your keyboard works.</source>
         <translation>您只需同时按两个键，或者连续按5次 Shift 键。这将关闭影响键盘工作方式的粘滞键功能。</translation>
     </message>
     <message>
-        <location filename="../../../plugins/xrandr/xrandr-manager.cpp" line="570"/>
         <source>The system detects that the HD device has been replaced.Do you need to switch to the recommended zoom (100%)? Click on the confirmation logout.</source>
-        <translation>系统检测到高清设备已被更换。您是否需要切换到建议的缩放比例（100%）？点击确认后会注销生效。</translation>
+        <translation type="vanished">系统检测到高清设备已被更换。您是否需要切换到建议的缩放比例（100%）？点击确认后会注销生效。</translation>
     </message>
     <message>
-        <location filename="../../../plugins/xrandr/xrandr-manager.cpp" line="575"/>
-        <location filename="../../../plugins/xrandr/xrandr-manager.cpp" line="600"/>
         <source>Scale tips</source>
-        <translation>缩放提示</translation>
+        <translation type="vanished">缩放提示</translation>
     </message>
     <message>
-        <location filename="../../../plugins/xrandr/xrandr-manager.cpp" line="578"/>
-        <location filename="../../../plugins/xrandr/xrandr-manager.cpp" line="603"/>
         <source>Confirmation</source>
-        <translation>确认</translation>
+        <translation type="vanished">确认</translation>
     </message>
     <message>
-        <location filename="../../../plugins/xrandr/xrandr-manager.cpp" line="579"/>
-        <location filename="../../../plugins/xrandr/xrandr-manager.cpp" line="604"/>
         <source>Cancel</source>
-        <translation>取消</translation>
+        <translation type="vanished">取消</translation>
     </message>
     <message>
-        <location filename="../../../plugins/xrandr/xrandr-manager.cpp" line="595"/>
         <source>Does the system detect high clear equipment and whether to switch to recommended scaling (200%)? Click on the confirmation logout.</source>
-        <translation>系统检测到高清设备，您是否切换到建议的缩放（200%）？点击确认后会注销生效。</translation>
+        <translation type="vanished">系统检测到高清设备，您是否切换到建议的缩放（200%）？点击确认后会注销生效。</translation>
     </message>
 </context>
 <context>
