@@ -16,6 +16,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+
+
 #include "clipboard-manager.h"
 #include <glib.h>
 #include <stdio.h>
@@ -43,7 +45,7 @@ ClipboardManager::ClipboardManager(QObject *parent) : QThread(parent)
     gdk_init(NULL, NULL);
     GdkDisplay* display = gdk_display_get_default();
     if (nullptr == display) {
-        CT_SYSLOG(LOG_ERR, "get GdkDisplay error");
+        USD_LOG(LOG_ERR, "get GdkDisplay error");
         return;
     }
 
@@ -52,6 +54,7 @@ ClipboardManager::ClipboardManager(QObject *parent) : QThread(parent)
 
 ClipboardManager::~ClipboardManager()
 {
+
 }
 
 void target_data_unref (TargetData *data)
@@ -107,7 +110,7 @@ void ClipboardManager::run()
         init_atoms (mDisplay);
         /* check if there is a clipboard manager running */
         if (XGetSelectionOwner (mDisplay, XA_CLIPBOARD_MANAGER)) {
-            CT_SYSLOG(LOG_ERR, "Clipboard manager is already running.");
+            USD_LOG(LOG_ERR, "Clipboard manager is already running.");
             mExit = false;
             return;
         }
