@@ -17,17 +17,41 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 #include <QDebug>
-#include "color-manager.h"
 #include <math.h>
+#include <QTime>
+#include <QDBusInterface>
+#include <QDBusArgument>
+#include <color-info.h>
+
+#include "color-manager.h"
 
 #define PLUGIN_COLOR_SCHEMA         "org.ukui.SettingsDaemon.plugins.color"
 #define COLOR_KEY_LAST_COORDINATES  "night-light-last-coordinates"
 #define COLOR_KEY_ENABLED           "night-light-enabled"
+#define COLOR_KEY_ALLDAY            "night-light-allday"
+#define COLOR_KEY_AUTO_THEME        "theme-schedule-automatic"
 #define COLOR_KEY_TEMPERATURE       "night-light-temperature"
 #define COLOR_KEY_AUTOMATIC         "night-light-schedule-automatic"
+#define COLOR_KEY_AUTOMATIC_FROM    "night-light-schedule-automatic-from"
+#define COLOR_KEY_AUTOMATIC_TO      "night-light-schedule-automatic-to"
 #define COLOR_KEY_FROM              "night-light-schedule-from"
 #define COLOR_KEY_TO                "night-light-schedule-to"
 
+#define GTK_THEME_SCHEMA            "org.mate.interface"
+#define GTK_THEME_KEY               "gtk-theme"
+
+#define QT_THEME_SCHEMA             "org.ukui.style"
+#define QT_THEME_KEY                "style-name"
+
+#define HAD_READ_KWIN               "had-read-kwin-config"
+
+#define KWIN_COLOR_ACTIVE            "Active"
+#define KWIN_NIGHT_TEMP              "NightTemperature"
+#define KWIN_COLOR_MODE              "Mode"
+#define KWIN_COLOR_START             "EveningBeginFixed"
+#define KWIN_COLOR_END               "MorningBeginFixed"
+#define KWIN_CURRENT_TEMP            "CurrentColorTemperature"
+
 #define USD_NIGHT_LIGHT_SCHEDULE_TIMEOUT    5       /* seconds */
 #define USD_NIGHT_LIGHT_POLL_TIMEOUT        60      /* seconds */
 #define USD_NIGHT_LIGHT_POLL_SMEAR          1       /* hours */
@@ -37,6 +61,18 @@
 #define USD_TEMPERATURE_MAX_DELTA           (10.f)
 
 #define DESKTOP_ID "ukui-color-panel"
+#include <sys/timerfd.h>
+#include <unistd.h>
+
+enum theme_status_switch{
+    white_mode,
+    black_mode
+};
+
+double hour_minute_to_value(int hour, int minute) {
+    double value = (double)minute/60;
+    return (double)hour + value;
+}
 
 ColorManager *ColorManager::mColorManager = nullptr;
 
@@ -53,32 +89,51 @@ ColorManager::ColorManager()
     cached_sunset   = -1.f;
     cached_temperature = USD_COLOR_TEMPERATURE_DEFAULT;
     settings = new QGSettings (PLUGIN_COLOR_SCHEMA);
+    gtk_settings = new QGSettings (GTK_THEME_SCHEMA);
+    qt_settings = new QGSettings (QT_THEME_SCHEMA);
     mColorState    = new ColorState();
     mColorProfiles = new ColorProfiles();
+    m_NightChecktimer  = new QTimer(this);
 }
 
 ColorManager::~ColorManager()
 {
-    if(mColorManager)
-        delete mColorManager;
-    if(settings)
+    if (m_NightChecktimer) {
+        delete m_NightChecktimer;
+        m_NightChecktimer = nullptr;
+    }
+    if (settings) {
         delete settings;
-    if(mColorState)
+        settings = nullptr;
+    }
+    if (gtk_settings) {
+        delete gtk_settings;
+        gtk_settings = nullptr;
+    }
+    if (qt_settings) {
+        delete qt_settings;
+        qt_settings = nullptr;
+    }
+    if (mColorState) {
         delete mColorState;
-    if(mColorProfiles)
+        mColorState = nullptr;
+    }
+    if (mColorProfiles) {
         delete mColorProfiles;
+        mColorProfiles = nullptr;
+    }
 }
 
 ColorManager *ColorManager::ColorManagerNew()
 {
-    if(nullptr == mColorManager)
+    if (nullptr == mColorManager)
         mColorManager = new ColorManager();
     return  mColorManager;
 }
 
 GDateTime *ColorManager::NightLightGetDateTimeNow()
 {
-    if(datetime_override != NULL)
+    if (datetime_override != NULL)
         return g_date_time_ref (datetime_override);
     return g_date_time_new_now_local();
 }
@@ -101,7 +156,7 @@ bool ColorManager::NightLightSmoothCb (ColorManager *manager)
         tmp *= frac;
         tmp += manager->cached_temperature;
         manager->NightLightSetTemperatureInternal (tmp);
-
+//        USD_LOG(LOG_DEBUG,"set Temp...%f",tmp);
         return G_SOURCE_CONTINUE;
 }
 
@@ -125,16 +180,20 @@ void ColorManager::PollSmoothDestroy ()
 
 void ColorManager::NightLightSetTemperatureInternal (double temperature)
 {
-        if (ABS (cached_temperature - temperature) <= USD_TEMPERATURE_MAX_DELTA)
-                return;
-        cached_temperature = temperature;
-        mColorState->ColorStateSetTemperature (cached_temperature);
+    if (ABS (cached_temperature - temperature) <= USD_TEMPERATURE_MAX_DELTA) {
+//        USD_LOG(LOG_DEBUG,"set night light %f error ABS:%f delta:%f", temperature,ABS (cached_temperature - temperature),USD_TEMPERATURE_MAX_DELTA);
+        return;
+    }
+
+    cached_temperature = temperature;
+    mColorState->ColorStateSetTemperature (cached_temperature);
 }
 
 void ColorManager::NightLightSetTemperature(double temperature)
 {
     /* immediate */
     if (!smooth_enabled) {
+            USD_LOG(LOG_DEBUG,"set night light %f", temperature);
             NightLightSetTemperatureInternal (temperature);
             return;
     }
@@ -150,17 +209,16 @@ void ColorManager::NightLightSetTemperature(double temperature)
 
     /* smooth out the transition */
     PollSmoothCreate (temperature);
+    USD_LOG(LOG_DEBUG,"set color temp to :%f",temperature);
 }
 
 void ColorManager::NightLightSetActive(bool active)
 {
-    if (cached_active == active)
-            return;
     cached_active = active;
-
     /* ensure set to unity temperature */
-    if (!active)
-            NightLightSetTemperature (USD_COLOR_TEMPERATURE_DEFAULT);
+    if (!active){
+        NightLightSetTemperature (USD_COLOR_TEMPERATURE_DEFAULT);
+    }
 }
 
 
@@ -319,13 +377,85 @@ bool ColorManager::UpdateCachedSunriseSunset()
     }
     return ret;
 }
+/*Active:1,使能，0禁用。
+ *0:全天，1跟随日出日落，2自定义
+ *Mode:1 自定义
+ *Mode:2--EveningBeginFixed（17:55:01）---跟随日出日落
+ *Mode:3--全天
+*/
+bool ColorManager::ReadKwinColorTempConfig()
+{
+    QVector<ColorInfo> nightColor;
+    if (settings->keys().contains(HAD_READ_KWIN)) {
+        if (settings->get(HAD_READ_KWIN).toBool() == true) {
+            USD_LOG(LOG_DEBUG,"Kwin had read over..");
+            return false;
+        }
+    } else {
+        USD_LOG(LOG_DEBUG,"can't find key:%s", HAD_READ_KWIN);
+        return false;
+    }
+
+    QDBusInterface colorIft("org.ukui.KWin",
+                            "/ColorCorrect",
+                            "org.ukui.kwin.ColorCorrect",
+                            QDBusConnection::sessionBus());
+
+    QDBusMessage result = colorIft.call("nightColorInfo");
+    const QDBusArgument &dbusArgs = result.arguments().at(0).value<QDBusArgument>().asVariant().value<QDBusArgument>();
+
+    dbusArgs.beginArray();
+    while (!dbusArgs.atEnd()) {
+        ColorInfo color;
+        dbusArgs >> color;
+        nightColor.push_back(color);
+    }
+    dbusArgs.endArray();
+
+    for (ColorInfo it : nightColor) {
+        mNightConfig.insert(it.arg, it.out.variant());
+    }
+
+    settings->set(COLOR_KEY_TEMPERATURE, mNightConfig[KWIN_NIGHT_TEMP].toInt());
+    settings->set(COLOR_KEY_ENABLED,mNightConfig[KWIN_COLOR_ACTIVE].toBool());
+
+    if (3 == mNightConfig[KWIN_COLOR_MODE].toInt()) {
+        settings->set(COLOR_KEY_ALLDAY, true);
+    } else if (2 == mNightConfig[KWIN_COLOR_MODE].toInt() && mNightConfig[KWIN_COLOR_START].toString() == "17:55:01"){
+        settings->set(COLOR_KEY_AUTOMATIC, true);
+    } else {
+        QTime startTime = QTime::fromString(mNightConfig[KWIN_COLOR_START].toString(),"hh:mm:ss");
+        QTime endTime = QTime::fromString(mNightConfig[KWIN_COLOR_END].toString(),"hh:mm:ss");
+
+        settings->set(COLOR_KEY_FROM, hour_minute_to_value(startTime.hour(), startTime.minute()));
+        settings->set(COLOR_KEY_TO, hour_minute_to_value(endTime.hour(), endTime.minute()));
+    }
+
+    USD_LOG_SHOW_PARAM1(mNightConfig[KWIN_COLOR_ACTIVE].toBool());
+    USD_LOG_SHOW_PARAM1(mNightConfig[KWIN_COLOR_MODE].toInt());
+    USD_LOG_SHOW_PARAMS(mNightConfig[KWIN_COLOR_START].toString().toLatin1().data());
+    USD_LOG_SHOW_PARAMS(mNightConfig[KWIN_COLOR_END].toString().toLatin1().data());
+
+    settings->set(HAD_READ_KWIN,true);
+    mNightConfig[KWIN_COLOR_ACTIVE] = false;
+    colorIft.call("setNightColorConfig", mNightConfig);
+
+    mNightConfig[KWIN_NIGHT_TEMP] = mNightConfig[KWIN_CURRENT_TEMP];
+    mNightConfig[KWIN_COLOR_ACTIVE] = false;
+    colorIft.call("setNightColorConfig", mNightConfig);
+//    USD_LOG(LOG_DEBUG,".");
+    return true;
+}
 
 void ColorManager::NightLightRecheck(ColorManager *manager)
 {
     double frac_day;
     double schedule_from = -1.f;
     double schedule_to = -1.f;
+    double theme_from = -1.f;
+    double theme_to = -1.f;
     double smear = USD_NIGHT_LIGHT_POLL_SMEAR; /* hours */
+    int theme_now = -1;
     guint temperature;
     guint temp_smeared;
     GDateTime *dt_now = manager->NightLightGetDateTimeNow ();
@@ -335,21 +465,60 @@ void ColorManager::NightLightRecheck(ColorManager *manager)
     if (manager->forced) {
         temperature = manager->settings->get(COLOR_KEY_TEMPERATURE).toUInt();
         manager->NightLightSetTemperature (temperature);
+        USD_LOG(LOG_DEBUG,"set color to :%d",temperature);
         return;
     }
 
+    /* calculate the position of the sun */
+    if (manager->settings->get(COLOR_KEY_AUTO_THEME).toBool()) {
+        manager->UpdateCachedSunriseSunset ();
+//        if (manager->cached_sunrise > 0.f && manager->cached_sunset > 0.f) {
+//            theme_to   = manager->cached_sunrise;
+//            theme_from = manager->cached_sunset;
+//        } else {
+//            theme_to = 7.0;
+//            theme_from = 18.0;
+//        }
+        theme_to = 7.0;
+        theme_from = 18.0;
+        /* get the current hour of a day as a fraction */
+        frac_day = NightLightFracDayFromDt (dt_now);
+
+        if(frac_day > theme_to && frac_day < theme_from)
+            theme_now = 0;
+        else
+            theme_now = 1;
+
+        if(theme_now) {//需要根据项目进行设置
+            manager->gtk_settings->set(GTK_THEME_KEY, "ukui-black-unity");
+            manager->qt_settings->set(QT_THEME_KEY, "ukui-dark");
+        } else {
+            manager->gtk_settings->set(GTK_THEME_KEY, "ukui-white-unity");
+            manager->qt_settings->set(QT_THEME_KEY, "ukui-light");
+        }
+    }
+
     if(!manager->settings->get(COLOR_KEY_ENABLED).toBool()){
-        qDebug("night light disabled, resetting");
+        USD_LOG(LOG_DEBUG, "stop it..");
         manager->NightLightSetActive (false);
         return;
     }
 
+    if(manager->settings->get(COLOR_KEY_ALLDAY).toBool()){
+        temperature = manager->settings->get(COLOR_KEY_TEMPERATURE).toUInt();
+        manager->NightLightSetTemperature (temperature);
+        USD_LOG(LOG_DEBUG,"COLOR_KEY_ALLDAY temperature:%d",temperature);
+        return;
+    }
+
     /* calculate the position of the sun */
     if (manager->settings->get(COLOR_KEY_AUTOMATIC).toBool()) {
         manager->UpdateCachedSunriseSunset ();
         if (manager->cached_sunrise > 0.f && manager->cached_sunset > 0.f) {
             schedule_to   = manager->cached_sunrise;
             schedule_from = manager->cached_sunset;
+            manager->settings->set(COLOR_KEY_AUTOMATIC_FROM, 18.00);
+            manager->settings->set(COLOR_KEY_AUTOMATIC_TO, 7.00);
         }
     }
 
@@ -361,8 +530,8 @@ void ColorManager::NightLightRecheck(ColorManager *manager)
 
     /* get the current hour of a day as a fraction */
     frac_day = NightLightFracDayFromDt (dt_now);
-    qDebug ("fractional day = %.3f, limits = %.3f->%.3f",
-         frac_day, schedule_from, schedule_to);
+//    //qDebug("fractional day = %.3f, limits = %.3f->%.3f",
+//         frac_day, schedule_from, schedule_to);
 
     /* disabled until tomorrow */
     if (manager->disabled_until_tmw) {
@@ -373,7 +542,7 @@ void ColorManager::NightLightRecheck(ColorManager *manager)
 
         /* Reset if disabled until tomorrow is more than 24h ago. */
         if (time_span > (GTimeSpan) 24 * 60 * 60 * 1000000) {
-            qDebug ("night light disabled until tomorrow is older than 24h, resetting disabled until tomorrow");
+            //qDebug("night light disabled until tomorrow is older than 24h, resetting disabled until tomorrow");
             reset = true;
         } else if (time_span > 0) {
             /* Or if a sunrise lies between the time it was disabled and now. */
@@ -383,7 +552,7 @@ void ColorManager::NightLightRecheck(ColorManager *manager)
                 NightLightFracDayIsBetween (schedule_to,
                                             frac_disabled,
                                             frac_day)) {
-                    qDebug ("night light sun rise happened, resetting disabled until tomorrow");
+                    //qDebug("night light sun rise happened, resetting disabled until tomorrow");
                     reset = true;
             }
         }
@@ -392,7 +561,7 @@ void ColorManager::NightLightRecheck(ColorManager *manager)
                 manager->disabled_until_tmw = false;
                 g_clear_pointer(&manager->disabled_until_tmw_dt, g_date_time_unref);
         } else {
-                qDebug ("night light still day-disabled, resetting");
+                USD_LOG(LOG_DEBUG,"night light still day-disabled, resetting");
                 manager->NightLightSetTemperature (USD_COLOR_TEMPERATURE_DEFAULT);
                 return;
         }
@@ -406,7 +575,7 @@ void ColorManager::NightLightRecheck(ColorManager *manager)
     if (!NightLightFracDayIsBetween (frac_day,
                                      schedule_from - smear,
                                      schedule_to)) {
-        g_debug ("not time for night-light");
+
         manager->NightLightSetActive (false);
         return;
     }
@@ -440,50 +609,11 @@ void ColorManager::NightLightRecheck(ColorManager *manager)
     } else {
         temp_smeared = temperature;
     }
-    qDebug ("night light mode on, using temperature of %uK (aiming for %uK)",
-         temp_smeared, temperature);
+//    //qDebug("night light mode on, using temperature of %uK (aiming for %uK)",
+//         temp_smeared, temperature);
     manager->NightLightSetActive (true);
     manager->NightLightSetTemperature (temp_smeared);
-}
-
-/* called when the time may have changed */
-bool ColorManager::NightLightRecheckCb(ColorManager *manager)
-{
-
-        /* recheck parameters, then reschedule a new timeout */
-        NightLightRecheck (manager);
-        PollTimeoutDestroy (manager);
-        PollTimeoutCreate (manager);
-
-        /* return value ignored for a one-time watch */
-        return G_SOURCE_REMOVE;
-}
-
-void ColorManager::PollTimeoutDestroy(ColorManager *manager)
-{
-    if (manager->source == NULL)
-                return;
-
-    g_source_destroy (manager->source);
-    g_source_unref (manager->source);
-    manager->source = NULL;
-}
-
-void ColorManager::PollTimeoutCreate(ColorManager *manager)
-{
-        g_autoptr(GDateTime) dt_now = NULL;
-        g_autoptr(GDateTime) dt_expiry = NULL;
-
-        if (manager->source != NULL)
-                return;
-
-        dt_now = manager->NightLightGetDateTimeNow ();
-        dt_expiry = g_date_time_add_seconds (dt_now, USD_NIGHT_LIGHT_POLL_TIMEOUT);
-
-        g_source_set_callback (manager->source,
-                               (GSourceFunc)NightLightRecheckCb,
-                               manager, NULL);
-        g_source_attach (manager->source, NULL);
+    USD_LOG(LOG_DEBUG,"set temp :%d",temp_smeared);
 }
 
 void ColorManager::OnLocationNotify(GClueSimple *simple,
@@ -503,7 +633,7 @@ void ColorManager::OnLocationNotify(GClueSimple *simple,
                           g_variant_new ("(dd)", latitude, longitude));
     g_clear_object(&setting);
 
-    qDebug ("got geoclue latitude %f, longitude %f", latitude, longitude);
+    // //qDebug("got geoclue latitude %f, longitude %f", latitude, longitude);
 
     /* recheck the levels if the location changed significantly */
     if (manager->UpdateCachedSunriseSunset ())
@@ -559,28 +689,49 @@ void ColorManager::StopGeoclue()
 
 void ColorManager::SettingsChangedCb(QString key)
 {
-    qDebug ("settings changed");
+//    //qDebug("settings changed");
+    if(key == COLOR_KEY_AUTOMATIC_FROM || key == COLOR_KEY_AUTOMATIC_TO){
+        return;
+    }
+
+    USD_LOG(LOG_DEBUG,"KEY:%s",key.toLatin1().data());
+
     NightLightRecheck(this);
-    //if(key == COLOR_KEY_TEMPERATURE){
     mColorState->ColorStateSetTemperature (cached_temperature);
-    //}
 }
 
 bool ColorManager::ColorManagerStart()
 {
-    qDebug()<<"Color manager start";
-    mColorProfiles->ColorProfilesStart();
-    mColorState->ColorStateStart();
-    NightLightRecheck(this);
-    PollTimeoutCreate(this);
-    StartGeoclue();
-    connect(settings,SIGNAL(changed(QString)),this,SLOT(SettingsChangedCb(QString)));
+    USD_LOG(LOG_DEBUG,"--Color manager start--");
+    int ms = 2000;
+    if (false == ReadKwinColorTempConfig()) {
+        ms = 100;
+    }
+
+    QTimer::singleShot(ms, this, [=](){
+        mColorProfiles->ColorProfilesStart();
+        mColorState->ColorStateStart();
+        NightLightRecheck(this);
+        connect(m_NightChecktimer, SIGNAL(timeout()), this, SLOT(checkTime()));
+        m_NightChecktimer->start(USD_NIGHT_LIGHT_POLL_TIMEOUT*1000);
+        StartGeoclue();
+        connect(settings,SIGNAL(changed(QString)),this,SLOT(SettingsChangedCb(QString)));
+    });
+
+
+
+
     return  true;
 }
 
+void ColorManager::checkTime()
+{
+    NightLightRecheck (this);
+}
+
 void ColorManager::ColorManagerStop()
 {
-    qDebug()<<"Color manager stop";
+    USD_LOG(LOG_DEBUG,"Color manager stop");
     mColorProfiles->ColorProfilesStop();
     mColorState->ColorStateStop();
     StopGeoclue();
