@@ -29,12 +29,14 @@ public:
     static PluginInterface *getInstance();
     virtual void activate();
     virtual void deactivate();
+    bool isTrialMode();
 
 private:
     HousekeepingPlugin();
     HousekeepingPlugin(HousekeepingPlugin&)=delete;
 
 private:
+    QString             userName;
     HousekeepingManager *mHouseManager;
     static PluginInterface     *mInstance;
 
