@@ -22,10 +22,7 @@
 #include <QObject>
 #include <QDialog>
 #include <QLabel>
-#include <QFile>
-#include <QProgressBar>
 #include <QPushButton>
-#include <QMessageBox>
 
 namespace Ui {
 class LdsmTrashEmpty;
@@ -38,16 +35,23 @@ class LdsmTrashEmpty : public QDialog
 public:
     explicit LdsmTrashEmpty(QWidget *parent = nullptr);
     ~LdsmTrashEmpty();
-    void trashEmptyStart();
-    void showConfirmationDialog();
+    void usdLdsmTrashEmpty();
+
+public Q_SLOTS:
+    void checkButtonCancel();
+    void checkButtonTrashEmpty();
 
 private:
     Ui::LdsmTrashEmpty *ui;
-
+    QLabel *first_text;
+    QLabel *second_text;
+    QPushButton *trash_empty;
+    QPushButton *cancel;
 
 private:
     void windowLayoutInit();
-
+    void connectEvent();
+    void deleteContents(const QString path);
 
 };
 
