@@ -22,9 +22,10 @@
 #include "qtimer.h"
 #include "syslog.h"
 
+
 #define GIGABYTE                   1024 * 1024 * 1024
 
-#define CHECK_EVERY_X_SECONDS      60
+#define CHECK_EVERY_X_SECONDS      120 * 1000
 
 #define DISK_SPACE_ANALYZER        "ukui-disk-usage-analyzer"
 
@@ -35,48 +36,37 @@
 #define SETTINGS_MIN_NOTIFY_PERIOD        "min-notify-period"
 #define SETTINGS_IGNORE_PATHS             "ignore-paths"
 
-//DIskSpace *DIskSpace::mDisk = nullptr;
 static guint64 *time_read;
-QTimer* DIskSpace::ldsm_timeout_cb = NULL;
-
-
-DIskSpace *DIskSpace::mDisk = NULL;
-GHashTable        *DIskSpace::ldsm_notified_hash  = NULL;
-QHash<const char*, LdsmMountInfo*> DIskSpace::m_notified_hash;
-
-
-GUnixMountMonitor *DIskSpace::ldsm_monitor = NULL;
-double             DIskSpace::free_percent_notify;
-double             DIskSpace::free_percent_notify_again;
-unsigned int       DIskSpace::free_size_gb_no_notify;
-unsigned int       DIskSpace::min_notify_period = 0;
-GSList            *DIskSpace::ignore_paths = NULL;
-QGSettings        *DIskSpace::settings;
-LdsmDialog        *DIskSpace::dialog = NULL;
 
 DIskSpace::DIskSpace()
 {
 
     ldsm_timeout_cb = new QTimer();
-    connect(ldsm_timeout_cb, SIGNAL(timeout()), this, SLOT(ldsm_check_all_mounts()));
-    ldsm_timeout_cb->start();
+    trash_empty=new LdsmTrashEmpty();
+    ldsm_notified_hash=NULL;
     ldsm_monitor = NULL;
     free_percent_notify = 0.05;
     free_percent_notify_again = 0.01;
     free_size_gb_no_notify = 2;
     min_notify_period = 10;
     ignore_paths = NULL;
+    done = FALSE;
+
+    connect(ldsm_timeout_cb, &QTimer::timeout,
+            this, &DIskSpace::ldsm_check_all_mounts);
+
+    ldsm_timeout_cb->start();
 
     if (QGSettings::isSchemaInstalled(SETTINGS_HOUSEKEEPING_SCHEMA)) {
         settings = new QGSettings(SETTINGS_HOUSEKEEPING_SCHEMA);
-
     }
-    dialog = NULL;
+    this->dialog = nullptr;
 }
 
 DIskSpace::~DIskSpace()
 {
-
+    delete trash_empty;
+    delete settings;
 }
 
 static gint
@@ -109,7 +99,7 @@ void DIskSpace::usdLdsmGetConfig()
     free_percent_notify =settings->get(SETTINGS_FREE_PC_NOTIFY_KEY).toDouble();
     if (free_percent_notify >= 1 || free_percent_notify < 0) {
         /* FIXME define min and max in gschema! */
-        g_warning ("Invalid configuration of free_percent_notify: %f\n" \
+        qWarning ("housekeeping: Invalid configuration of free_percent_notify: %f\n" \
                    "Using sensible default", free_percent_notify);
         free_percent_notify = 0.05;
     }
@@ -117,7 +107,7 @@ void DIskSpace::usdLdsmGetConfig()
     free_percent_notify_again = settings->get(SETTINGS_FREE_PC_NOTIFY_AGAIN_KEY).toDouble();
     if (free_percent_notify_again >= 1 || free_percent_notify_again < 0) {
         /* FIXME define min and max in gschema! */
-        g_warning ("Invalid configuration of free_percent_notify_again: %f\n" \
+        qWarning ("housekeeping: Invalid configuration of free_percent_notify_again: %f\n" \
                    "Using sensible default\n", free_percent_notify_again);
         free_percent_notify_again = 0.01;
     }
@@ -132,15 +122,6 @@ void DIskSpace::usdLdsmGetConfig()
         g_slist_free (ignore_paths);
         ignore_paths = NULL;
     }
-
-    // 取得清理忽略的目录
-    //settings_list =settings->getStrv(SETTINGS_IGNORE_PATHS);
-    QVariantList ignoreList = settings->choices(SETTINGS_IGNORE_PATHS);
-    QVariantList::const_iterator it;
-    for (it = ignoreList.constBegin(); it != ignoreList.constEnd(); ++it) {
-        m_notified_hash.remove((*it).toString().toLatin1().data());
-    }
-
 }
 
 static void
@@ -169,7 +150,6 @@ static gboolean is_hash_item_not_in_mounts(QHash<const char*, LdsmMountInfo*> &h
         if (hash.find(path) !=hash.end() )
             return FALSE;
     }
-
     return TRUE;
 }
 
@@ -190,7 +170,7 @@ bool DIskSpace::ldsm_mount_should_ignore (GUnixMountEntry *mount)
     const gchar *fs, *device, *path;
 
     path = g_unix_mount_get_mount_path (mount);
-    if (DIskSpace::ldsm_mount_is_user_ignore (path))
+    if (ldsm_mount_is_user_ignore (path))
         return TRUE;
 
     /* This is borrowed from GLib and used as a way to determine
@@ -262,17 +242,19 @@ bool DIskSpace::ldsm_mount_should_ignore (GUnixMountEntry *mount)
 bool  DIskSpace::ldsm_mount_has_space (LdsmMountInfo *mount)
 {
     double free_space;
-
+    bool percent_flag = false;
+    bool size_falg = false;
     free_space = (double) mount->buf.f_bavail / (double) mount->buf.f_blocks;
+
     /* enough free space, nothing to do */
     if (free_space > free_percent_notify)
-        return true;
+        percent_flag = true;
 
     if (((gint64) mount->buf.f_frsize * (gint64) mount->buf.f_bavail) > ((gint64) free_size_gb_no_notify * GIGABYTE))
-        return true;
+        size_falg = true;
 
-    /* If we got here, then this volume is low on space */
-    return false;
+    /* If it returns false, then this volume is low on space */
+    return (percent_flag && size_falg);
 }
 
 static bool ldsm_mount_is_virtual (LdsmMountInfo *mount)
@@ -281,7 +263,6 @@ static bool ldsm_mount_is_virtual (LdsmMountInfo *mount)
         /* Filesystems with zero blocks are virtual */
         return true;
     }
-
     return false;
 }
 
@@ -373,10 +354,12 @@ static gboolean ldsm_mount_has_trash (LdsmMountInfo *mount)
 
 static void ldsm_analyze_path (const gchar *path)
 {
+
     const gchar *argv[] = { DISK_SPACE_ANALYZER, path, NULL };
 
     g_spawn_async (NULL, (gchar **) argv, NULL, G_SPAWN_SEARCH_PATH,
                    NULL, NULL, NULL, NULL);
+
 }
 
 bool DIskSpace::ldsm_notify_for_mount (LdsmMountInfo *mount,
@@ -411,31 +394,34 @@ bool DIskSpace::ldsm_notify_for_mount (LdsmMountInfo *mount,
                              free_space,
                              name,
                              path);
-
     g_free (name);
 
+//    dialog->show();
     response = dialog->exec();
     delete dialog;
+    dialog = nullptr;
 
     switch (response) {
     case GTK_RESPONSE_CANCEL:
-        retval = FALSE;
+        retval = false;
         break;
     case LDSM_DIALOG_RESPONSE_ANALYZE:
-        retval = FALSE;
+        retval = false;
         ldsm_analyze_path (path);
         break;
     case LDSM_DIALOG_RESPONSE_EMPTY_TRASH:
-        retval = TRUE;
-        // usd_ldsm_trash_empty ();//调清空回收站dialog
+        retval = false;
+        trash_empty->usdLdsmTrashEmpty();//调清空回收站dialog
         break;
-
     case GTK_RESPONSE_NONE:
     case GTK_RESPONSE_DELETE_EVENT:
-        retval = TRUE;
+        retval = true;
+        break;
+    case LDSM_DIALOG_IGNORE:
+        retval = true;
         break;
     default:
-        g_assert_not_reached ();
+        retval = false;
     }
     free (path);
     return retval;
@@ -447,7 +433,6 @@ void DIskSpace::ldsm_maybe_warn_mounts (GList *mounts,
                                         bool other_usable_volumes)
 {
     GList *l;
-    gboolean done = FALSE;
 
     for (l = mounts; l != NULL; l = l->next) {
         LdsmMountInfo *mount_info = (LdsmMountInfo  *)l->data;
@@ -457,6 +442,8 @@ void DIskSpace::ldsm_maybe_warn_mounts (GList *mounts,
         time_t curr_time;
         const char *path;
         gboolean show_notify;
+        QString every_two_minutes_check;
+        gdouble free_space_check;
 
         if (done) {
             /* Don't show any more dialogs if the user took action with the last one. The user action
@@ -468,9 +455,9 @@ void DIskSpace::ldsm_maybe_warn_mounts (GList *mounts,
 
         path = g_unix_mount_get_mount_path (mount_info->mount);
 
-                previous_mount_info = (LdsmMountInfo *)g_hash_table_lookup (ldsm_notified_hash, path);
-                if (previous_mount_info != NULL)
-                    previous_free_space = (gdouble) previous_mount_info->buf.f_bavail / (gdouble) previous_mount_info->buf.f_blocks;
+        previous_mount_info = (LdsmMountInfo *)g_hash_table_lookup (ldsm_notified_hash, path);
+        if (previous_mount_info != NULL)
+            previous_free_space = (gdouble) previous_mount_info->buf.f_bavail / (gdouble) previous_mount_info->buf.f_blocks;
         //        liutong
         // 找m_notified_hash中(mount_info->mount)得到的path
         QHash<const char*, LdsmMountInfo*>::iterator it = m_notified_hash.find(path);
@@ -494,7 +481,7 @@ void DIskSpace::ldsm_maybe_warn_mounts (GList *mounts,
             if (difftime (curr_time, previous_mount_info->notify_time) > (gdouble)(min_notify_period * 60)) {
                 show_notify = TRUE;
                 mount_info->notify_time = curr_time;
-            } else {
+            }else {
                 /* It's too soon to show the dialog again. However, we still replace the LdsmMountInfo
                  * struct in the hash table, but give it the notfiy time from the previous dialog.
                  * This will stop the notification from reappearing unnecessarily as soon as the timeout expires.
@@ -513,9 +500,25 @@ void DIskSpace::ldsm_maybe_warn_mounts (GList *mounts,
             if (ldsm_notify_for_mount (mount_info, multiple_volumes, other_usable_volumes))
                 done = TRUE;
         }
+
+        free_space_check = (gint64) mount_info->buf.f_frsize * (gint64) mount_info->buf.f_bavail;
+        every_two_minutes_check = QString().sprintf("The volume \"%1\" has %s disk space remaining.",
+                                       g_format_size(free_space_check)).arg(g_unix_mount_guess_name(mount_info->mount));
+        printf("%s\n",every_two_minutes_check.toStdString().data());
     }
 }
 
+bool DIskSpace::ldsmGetIgnorePath(const gchar *path)
+{
+    // 取得清理忽略的目录
+    QStringList ignoreList = settings->get(SETTINGS_IGNORE_PATHS).toStringList();
+    for (QString it : ignoreList) {
+        if (it.compare(path) == 0)
+            return true;
+    }
+    return false;
+}
+
 bool DIskSpace::ldsm_check_all_mounts ()
 {
     GList *mounts;
@@ -531,6 +534,7 @@ bool DIskSpace::ldsm_check_all_mounts ()
      * Iterating through the static mounts means we automatically ignore dynamically mounted media.
      */
     ldsm_timeout_cb->stop();
+    ldsm_timeout_cb->start(CHECK_EVERY_X_SECONDS);
     mounts = g_unix_mount_points_get (time_read);
 
     for (l = mounts; l != NULL; l = l->next) {
@@ -551,6 +555,18 @@ bool DIskSpace::ldsm_check_all_mounts ()
         mount_info->mount = mount;
 
         path = g_unix_mount_get_mount_path (mount);
+
+        if (g_strcmp0 (path, "/boot/efi") == 0 ||
+            g_strcmp0 (path, "/boot") == 0 ){
+            ldsm_free_mount_info (mount_info);
+            continue;
+        }
+
+        if (ldsmGetIgnorePath(path)){
+            ldsm_free_mount_info (mount_info);
+            continue;
+        }
+
         if (g_unix_mount_is_readonly (mount)) {
             ldsm_free_mount_info (mount_info);
             continue;
@@ -601,43 +617,40 @@ bool DIskSpace::ldsm_check_all_mounts ()
     return TRUE;
 }
 
-void DIskSpace::ldsm_mounts_changed (GObject  *monitor,gpointer  data)
+void DIskSpace::ldsm_mounts_changed (GObject  *monitor,gpointer  data,DIskSpace *disk)
 {
     GList *mounts;
-
     /* remove the saved data for mounts that got removed */
     mounts = g_unix_mounts_get (time_read);
 
-    is_hash_item_not_in_mounts(m_notified_hash, mounts);
+    is_hash_item_not_in_mounts(disk->m_notified_hash, mounts);
     g_list_free_full (mounts, (GDestroyNotify) g_unix_mount_free);
 
     /* check the status now, for the new mounts */
-    ldsm_check_all_mounts ();
-
-    /* and reset the timeout */
-    ldsm_timeout_cb->start(CHECK_EVERY_X_SECONDS);
+    disk->ldsm_check_all_mounts();
 }
 
 void DIskSpace::UsdLdsmSetup(bool check_now)
 {
     if (!m_notified_hash.empty() || ldsm_timeout_cb || ldsm_monitor) {
         qWarning ("Low disk space monitor already initialized.");
-        return;
+        //return;
     }
     usdLdsmGetConfig();
-    connect(settings,SIGNAL(changes(QString)),this,SLOT(usdLdsmUpdateConfig(QString)));
+    connect(settings, &QGSettings::changed,
+            this,    &DIskSpace::usdLdsmUpdateConfig);
+
 #if GLIB_CHECK_VERSION (2, 44, 0)
     ldsm_monitor = g_unix_mount_monitor_get ();
 #else
     ldsm_monitor = g_unix_mount_monitor_new ();
     g_unix_mount_monitor_set_rate_limit (ldsm_monitor, 1000);
 #endif
-    g_signal_connect (ldsm_monitor, "mounts-changed",
-                      G_CALLBACK (DIskSpace::ldsm_mounts_changed), NULL);
+    /*g_signal_connect (ldsm_monitor, "mounts-changed",
+                      G_CALLBACK (DIskSpace::ldsm_mounts_changed), this);*/
     if (check_now)
         ldsm_check_all_mounts ();
     //ldsm_timeout_cb->start(CHECK_EVERY_X_SECONDS);
-
 }
 
 void DIskSpace::cleanNotifyHash() {
@@ -652,7 +665,6 @@ void DIskSpace::cleanNotifyHash() {
 
 void DIskSpace::UsdLdsmClean()
 {
-
     cleanNotifyHash();
 
     if (ldsm_monitor)
@@ -662,12 +674,6 @@ void DIskSpace::UsdLdsmClean()
     if (settings) {
         g_object_unref (settings);
     }
-    /*
-    if (dialog) {
-        delete dialog;
-        dialog = NULL;
-    }
-*/
     if (ignore_paths) {
         g_slist_foreach (ignore_paths, (GFunc) g_free, NULL);
         g_slist_free (ignore_paths);
