@@ -18,6 +18,7 @@
  */
 #include "mouse-manager.h"
 #include "clib-syslog.h"
+#include "usd_base_class.h"
 
 /* Keys with same names for both touchpad and mouse */
 #define KEY_LEFT_HANDED                  "left-handed"          /*  a boolean for mouse, an enum for touchpad */
@@ -52,6 +53,11 @@
 #define KEY_TOUCHPAD_BOTTOM_R_C_CLICK_M  "bottom-right-corner-click-menu"   //右下角点击菜单	true/false
 #define KEY_TOUCHPAD_MOUSE_SENSITVITY    "mouse-sensitivity"                //鼠标敏感度	1-4  四个档位  低中高最高
 
+
+extern "C"{
+#include <X11/extensions/XInput2.h>
+}
+
 typedef enum {
         TOUCHPAD_HANDEDNESS_RIGHT,
         TOUCHPAD_HANDEDNESS_LEFT,
@@ -97,14 +103,14 @@ MouseManager * MouseManager::MouseManagerNew()
 
 bool MouseManager::MouseManagerStart()
 {
-    CT_SYSLOG(LOG_DEBUG,"-- Mouse Start Manager --");
+    USD_LOG(LOG_DEBUG,"-- Mouse Start Manager --");
 
     if (!supports_xinput_devices()){
         qWarning("XInput is not supported, not applying any settings");
         return TRUE;
     }
     time = new QTimer(this);
-    connect(time,SIGNAL(timeout()),this,SLOT(MouseManagerIdleCb()));
+    connect(time, &QTimer::timeout, this, &MouseManager::MouseManagerIdleCb);
     time->start();
     return true;
 }
@@ -112,7 +118,7 @@ bool MouseManager::MouseManagerStart()
 void MouseManager::MouseManagerStop()
 {
 
-    syslog(LOG_DEBUG,"-- Stopping Mouse Manager --");
+    USD_LOG(LOG_DEBUG,"-- Stopping Mouse Manager --");
 
     SetLocatePointer(FALSE);
 
@@ -239,6 +245,51 @@ Atom property_from_name (const char *property_name)
     return XInternAtom (QX11Info::display(), property_name, True);
 }
 
+bool query_device_had_property(XDeviceInfo *device_info,const char *property_name)
+{
+    XDevice *device;
+    int nProps = 0;
+    Atom* type_data;
+    unsigned char *data;
+    Atom Xdata;
+    bool ret = false;
+    Display * display = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
+    device = XOpenDevice (display, device_info->id);
+    USD_LOG(LOG_DEBUG,"device name：%s",device_info->name);
+
+    if (device == NULL) {
+        USD_LOG(LOG_DEBUG, "%s find %s had a error:can't open id(%d) device in XOpenDevice,type：%d",device_info->name, property_name,device_info->id,device_info->type);
+        return false;
+    }
+
+    type_data =  XListDeviceProperties(display ,device, &nProps);
+    Xdata = XInternAtom (QX11Info::display(), property_name, True);
+
+    //USD_LOG_SHOW_PARAM1(nProps);
+    //USD_LOG_SHOW_PARAM1(Xdata);
+    //USD_LOG_SHOW_PARAMS(property_name);
+
+    if (Xdata == 0) {
+        ret = false;
+        goto END;
+    }
+
+    for (int k = 0; k < nProps; k++) {
+//        USD_LOG_SHOW_PARAM1(type_data[k]);
+        if (Xdata == type_data[k]) {
+            USD_LOG(LOG_DEBUG,"find: props");
+            //USD_LOG_SHOW_PARAM1(type_data[k]);
+            ret = true;
+            goto END;
+        }
+    }
+
+END:
+    XCloseDevice (display, device);
+    return ret;
+}
+
+/* This is an example of the last function.
 bool property_exists_on_device (XDeviceInfo *device_info, const char  *property_name)
 {
     XDevice *device;
@@ -247,27 +298,36 @@ bool property_exists_on_device (XDeviceInfo *device_info, const char  *property_
     int format;
     unsigned long nitems, bytes_after;
     unsigned char *data;
-    Display *display = gdk_x11_get_default_xdisplay ();//QX11Info::display();
+   // Display *display = gdk_x11_get_default_xdisplay ();//QX11Info::display();
+    Display * display = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
     prop = property_from_name (property_name);
+
     if (!prop)
             return FALSE;
     try {
         device = XOpenDevice (display, device_info->id);
-        if (device == NULL)
+
+        if (device == NULL) {
+            USD_LOG(LOG_DEBUG, "%s find %s had a error:can't open id(%d) device in XOpenDevice,type：%d",device_info->name, property_name,device_info->id,device_info->type);
             throw 1;
+        }
+//        USD_LOG(LOG_DEBUG,"prop [%s] had find in %d type:%d", property_name, device_info->id, device_info->type);
         rc = XGetDeviceProperty (display,
                                  device, prop, 0, 1, False, XA_INTEGER, &type, &format,
                                  &nitems, &bytes_after, &data);
+        USD_LOG_SHOW_PARAM1(rc);
         if (rc == Success)
                 XFree (data);
 
         XCloseDevice (display, device);
     } catch (int str) {
-        CT_SYSLOG(LOG_DEBUG,"MOUSE: WRING ID: %d",str);
+        USD_LOG(LOG_DEBUG,"MOUSE: WRING ID: %d",str);
         return FALSE;
     }
+
     return rc == Success;
 }
+*/
 
 void property_set_bool (XDeviceInfo *device_info,
                         XDevice     *device,
@@ -300,12 +360,12 @@ void property_set_bool (XDeviceInfo *device_info,
         }
         if (rc == Success)
                 XFree (data);
+
         if(gdk_x11_display_error_trap_pop (gdk_display_get_default()))
-            qWarning ("Error while setting %s on \"%s\"", property_name, device_info->name);
+             USD_LOG(LOG_DEBUG,"Error while setting %s on \"%s\"", property_name, device_info->name);
 
     } catch (int x) {
-        CT_SYSLOG(LOG_DEBUG,"MOUSE:Error while setting %s on \"%s\"", property_name, device_info->name)
-
+        USD_LOG(LOG_DEBUG,"MOUSE:Error while setting %s on \"%s\"", property_name, device_info->name)
     }
 }
 
@@ -420,8 +480,8 @@ void set_tap_to_click_synaptics (XDeviceInfo *device_info,
                 one_finger_tap = 1;
         if (two_finger_tap > 3 || two_finger_tap < 1)
                 two_finger_tap = 3;
-        if (three_finger_tap > 3 || three_finger_tap < 1)
-                three_finger_tap = 2;
+        if (three_finger_tap > 3 || three_finger_tap < 0)
+                three_finger_tap = 0;
 
         if (rc == Success && type == XA_INTEGER && format == 8 && nitems >= 7)
         {
@@ -436,6 +496,8 @@ void set_tap_to_click_synaptics (XDeviceInfo *device_info,
         if (rc == Success)
                 XFree (data);
 
+        USD_LOG(LOG_DEBUG,"...");
+
         XCloseDevice (display, device);
     } catch (int x) {
         qWarning("Error in setting tap to click on \"%s\"", device_info->name);
@@ -526,6 +588,8 @@ void MouseManager::SetLeftHandedLegacyDriver (XDeviceInfo     *device_info,
                     int one_finger_tap = settings_touchpad->get(KEY_TOUCHPAD_ONE_FINGER_TAP).toInt();
                     int two_finger_tap = settings_touchpad->get(KEY_TOUCHPAD_TWO_FINGER_TAP).toInt();
                     int three_finger_tap = settings_touchpad->get(KEY_TOUCHPAD_THREE_FINGER_TAP).toInt();
+
+                    USD_LOG_SHOW_PARAM1(left_handed);
                     set_tap_to_click_synaptics (device_info, tap, left_handed, one_finger_tap, two_finger_tap, three_finger_tap);
             }
 
@@ -534,6 +598,9 @@ void MouseManager::SetLeftHandedLegacyDriver (XDeviceInfo     *device_info,
                     return;
     } else {
             left_handed = mouse_left_handed;
+            USD_LOG(LOG_DEBUG,"SET IT");
+            USD_LOG_SHOW_PARAM1(left_handed);
+            //USD_LOG(LOG_DEBUG,".");
     }
 
     try {
@@ -564,7 +631,7 @@ void MouseManager::SetLeftHandedLegacyDriver (XDeviceInfo     *device_info,
 
         g_free (buttons);
     } catch (int x) {
-        CT_SYSLOG(LOG_DEBUG,"MOUSE :error id %d",x);
+        USD_LOG(LOG_DEBUG,"MOUSE :error id %d",x);
         return;
     }
 }
@@ -573,10 +640,13 @@ void MouseManager::SetLeftHanded (XDeviceInfo  *device_info,
                       bool         mouse_left_handed,
                       bool         touchpad_left_handed)
 {
-    if (property_exists_on_device (device_info, "libinput Left Handed Enabled"))
+    if (query_device_had_property (device_info, "libinput Left Handed Enabled")){
         set_left_handed_libinput (device_info, mouse_left_handed, touchpad_left_handed);
-    else
+        USD_LOG_SHOW_PARAM1(mouse_left_handed);
+    } else {
+        USD_LOG_SHOW_PARAM1(touchpad_left_handed);
         SetLeftHandedLegacyDriver (device_info, mouse_left_handed, touchpad_left_handed);
+    }
 }
 
 void MouseManager::SetLeftHandedAll (bool mouse_left_handed,
@@ -598,7 +668,7 @@ void MouseManager::SetLeftHandedAll (bool mouse_left_handed,
         XFreeDeviceList (device_info);
 }
 
-void MouseManager::SetMotionLibinput (XDeviceInfo     *device_info)
+void MouseManager::SetMotionLibinput (XDeviceInfo *device_info)
 {
     XDevice *device;
     Atom prop;
@@ -628,7 +698,7 @@ void MouseManager::SetMotionLibinput (XDeviceInfo     *device_info)
     try {
         device = device_is_touchpad (device_info);
         if (device != NULL) {
-            qDebug()<<"device != NULL  settings = settings_touchpad";
+            USD_LOG(LOG_DEBUG,"device != NULL  settings = settings_touchpad");
             settings = settings_touchpad;
         } else {
             device = XOpenDevice (dpy, device_info->id);
@@ -693,7 +763,7 @@ void MouseManager::SetMotionLegacyDriver (XDeviceInfo     *device_info)
             if (device == NULL)
                 throw 1;
         } catch (int x) {
-            CT_SYSLOG(LOG_DEBUG,"%s: error id %d","MOUSE", x);
+            USD_LOG(LOG_DEBUG,"%s: error id %d","MOUSE", x);
             return;
         }
             settings = settings_mouse;
@@ -729,7 +799,8 @@ void MouseManager::SetMotionLegacyDriver (XDeviceInfo     *device_info)
 
     /* And threshold */
     motion_threshold = settings->get(KEY_MOTION_THRESHOLD).toInt();
-    qDebug()<<__func__<<" motion_threshold = "<<motion_threshold;
+//    qDebug()<<__func__<<" motion_threshold = "<<motion_threshold;
+    USD_LOG(LOG_DEBUG,"motion_threshold:%d", motion_threshold);
     /* Get the list of feedbacks for the device */
     states = XGetFeedbackControl (dpy, device, &num_feedbacks);
     if (states == NULL) {
@@ -748,7 +819,7 @@ void MouseManager::SetMotionLegacyDriver (XDeviceInfo     *device_info)
             feedback.accelNum   = numerator;
             feedback.accelDenom = denominator;
 
-            qDebug ("Setting accel %d/%d, threshold %d for device '%s'",
+            USD_LOG(LOG_DEBUG,"Setting accel %d/%d, threshold %d for device '%s'",
                      numerator, denominator, motion_threshold, device_info->name);
 
             XChangeFeedbackControl (dpy,
@@ -773,8 +844,8 @@ void MouseManager::SetTouchpadMotionAccel(XDeviceInfo *device_info)
     unsigned long nitems, bytes_after;
 
 
-    Display * dpy = gdk_x11_get_default_xdisplay ();//QX11Info::display();
-
+//    Display * dpy = gdk_x11_get_default_xdisplay ();//QX11Info::display();
+    Display * dpy = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());//Display * dpy = QX11Info::display();
     union {
         unsigned char *c;
         long *l;
@@ -803,7 +874,8 @@ void MouseManager::SetTouchpadMotionAccel(XDeviceInfo *device_info)
 
         rc = XGetDeviceProperty (dpy,device, prop, 0, 1, False, float_type, &type,
                                  &format, &nitems, &bytes_after, &data.c);
-        qDebug()<<"format = "<<format<<"nitems = "<<nitems;
+
+        USD_LOG_SHOW_PARAM2(format,nitems)
         if (rc == Success && type == float_type && format == 32 && nitems >= 1) {
                 *(float *) data.l = accel;
                 XChangeDeviceProperty (dpy, device, prop, float_type, 32,
@@ -815,7 +887,7 @@ void MouseManager::SetTouchpadMotionAccel(XDeviceInfo *device_info)
         XCloseDevice (dpy, device);
 
     } catch (int x) {
-        qWarning("%s Error while setting accel speed on \"%s\"", device_info->name);
+        USD_LOG(LOG_ERR,"catch a bug...");
         return;
     }
 }
@@ -827,7 +899,8 @@ void MouseManager::SetMouseAccel(XDeviceInfo *device_info)
     int format, rc;
     unsigned long nitems, bytes_after;
 
-    Display * dpy = QX11Info::display();
+    //Display * dpy = QX11Info::display();
+    Display * dpy = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
     unsigned char *data;
     bool MouseAccel;
 
@@ -861,27 +934,32 @@ void MouseManager::SetMouseAccel(XDeviceInfo *device_info)
         XCloseDevice (dpy, device);
 
     } catch (int x) {
-        qWarning("%s Error while setting accel speed on \"%s\"", device_info->name);
+        USD_LOG(LOG_ERR,"catch a bug...");
+//        qWarning("%s Error while setting accel speed on \"%s\"", device_info->name);
         return;
     }
 }
 
-void MouseManager::SetMotion (XDeviceInfo    *device_info)
+void MouseManager::SetMotion (XDeviceInfo *device_info)
 {
-    if (property_exists_on_device (device_info, "libinput Accel Speed"))
+    if (query_device_had_property (device_info, "libinput Accel Speed")) {
+
         SetMotionLibinput (device_info);
-    else
+    }
+    else {
         SetMotionLegacyDriver (device_info);
+    }
+    if(query_device_had_property (device_info, "Device Accel Constant Deceleration")) {
 
-    if(property_exists_on_device (device_info, "Device Accel Constant Deceleration"))
         SetTouchpadMotionAccel(device_info);
+    }
+    if(query_device_had_property (device_info, "libinput Accel Profile Enabled")) {
 
-    if(property_exists_on_device (device_info, "libinput Accel Profile Enabled")) {
         SetMouseAccel(device_info);
     }
 }
 
-void MouseManager::SetMotionAll ()
+void MouseManager::SetMotionAll()
 {
     XDeviceInfo *device_info = nullptr;
     int n_devices;
@@ -910,7 +988,8 @@ void set_middle_button_evdev (XDeviceInfo *device_info,
     unsigned long nitems, bytes_after;
     unsigned char *data;
 
-    Display * display = QX11Info::display();
+    Display * display = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
+   // Display * display = QX11Info::display();
     prop = property_from_name ("Evdev Middle Button Emulation");
     if (!prop) /* no evdev devices */
         return;
@@ -958,7 +1037,7 @@ void set_middle_button_libinput (XDeviceInfo *device_info,
         property_set_bool (device_info, device, "libinput Middle Emulation Enabled", 0, middle_button);
         XCloseDevice (display, device);
     } catch (int x) {
-        CT_SYSLOG(LOG_DEBUG,"%s:error id %d","MOUSE",x);
+        USD_LOG(LOG_DEBUG,"%s:error id %d","MOUSE",x);
         return;
     }
 }
@@ -1044,7 +1123,9 @@ void MouseManager::SetMouseWheelSpeed (int speed)
                    "Shift_L,   Down, Shift_L|Button5\n"
                    "Shift_R,   Down, Shift_R|Button5\n"
                    "None,      Up,   Button4, %1, 0, %2\n"
-                   "None,      Down, Button5, %3, 0, %4\n")
+                   "None,      Down, Button5, %3, 0, %4\n"
+                   "None,      Thumb1,  Alt_L|Left\n"
+                   "None,      Thumb2,  Alt_L|Right")
                 .arg(speed).arg(delay).arg(speed).arg(delay);
 
     file.setFileName(FilePath);
@@ -1057,12 +1138,12 @@ void MouseManager::SetMouseWheelSpeed (int speed)
     char **args;
     int    argc;
 
-    if (imwheelSpawned){
-        QProcess::execute("killall imwheel");
-        imwheelSpawned = false;
-    }
+//    if (imwheelSpawned){
+//        QProcess::execute("killall imwheel");
+//        imwheelSpawned = false;
+//    }
 
-    QString str = "/usr/bin/imwheel";
+    QString str = "/usr/bin/imwheel -k";
     if( g_shell_parse_argv (str.toLatin1().data(), &argc, &args, NULL)){
         g_spawn_async (g_get_home_dir (),
                        args,
@@ -1085,7 +1166,8 @@ void MouseManager::MouseCallback (QString keys)
         bool mouse_left_handed = settings_mouse->get(keys).toBool();
         bool touchpad_left_handed = GetTouchpadHandedness (mouse_left_handed);
         SetLeftHandedAll (mouse_left_handed, touchpad_left_handed);
-
+        USD_LOG(LOG_DEBUG,"...............");
+        SetTapToClickAll ();
     } else if ((keys.compare(QString::fromLocal8Bit(KEY_MOTION_ACCELERATION))==0) ||
                (keys.compare(QString::fromLocal8Bit(KEY_MOTION_THRESHOLD))==0) ||
                (keys.compare(QString::fromLocal8Bit(KEY_MOUSE_ACCEL)) == 0)){
@@ -1097,6 +1179,8 @@ void MouseManager::MouseCallback (QString keys)
         SetLocatePointer (settings_mouse->get(keys).toBool());
     } else if(keys.compare(QString::fromLocal8Bit(KEY_MOUSE_WHEEL_SPEED)) == 0 ) {
         SetMouseWheelSpeed (settings_mouse->get(keys).toInt());
+    }else{
+        USD_LOG(LOG_DEBUG,"keys:is skip..k%s", keys.toLatin1().data());
     }
 }
 
@@ -1111,18 +1195,26 @@ gboolean have_program_in_path (const char *name)
     return result;
 }
 
-void MouseManager::SetDisableWTypingSynaptics (bool         state)
+void MouseManager::SetDisableWTypingSynaptics (bool state)
 {
     if (state && touchpad_is_present ()) {
         GError *error = NULL;
         char **args;
         int    argc;
-        QString cmd = "syndaemon -i 0.5 -K -R";
-        if (syndaemon_spawned)
+        QString cmd = "syndaemon -i 0.3 -K -R";
+
+        if (syndaemon_spawned) {
+            kill (syndaemon_pid, SIGHUP);
+            g_spawn_close_pid (syndaemon_pid);
+            syndaemon_spawned = FALSE;
+            USD_LOG(LOG_DEBUG,"stop syndaemon");
+        }
+
+
+        if (!have_program_in_path ("syndaemon")) {
             return;
+        }
 
-        if (!have_program_in_path ("syndaemon"))
-                return;
         if (g_shell_parse_argv (cmd.toLatin1().data(), &argc, &args, NULL)) {
             g_spawn_async (g_get_home_dir (),
                            args,
@@ -1134,14 +1226,17 @@ void MouseManager::SetDisableWTypingSynaptics (bool         state)
                            &error);
             syndaemon_spawned = (error == NULL);
         }
+        USD_LOG(LOG_DEBUG,"start syndaemon(%d)", syndaemon_pid);
         if (error) {
                 settings_touchpad->set(KEY_TOUCHPAD_DISABLE_W_TYPING,false);
+                USD_LOG(LOG_ERR,"find error %s",error->message);
                 g_error_free (error);
+
         }
+
         g_strfreev (args);
 
-    } else if (syndaemon_spawned)
-    {
+    } else if (syndaemon_spawned) {
         kill (syndaemon_pid, SIGHUP);
         g_spawn_close_pid (syndaemon_pid);
         syndaemon_spawned = FALSE;
@@ -1188,7 +1283,7 @@ void MouseManager::SetDisableWTypingLibinput (bool  state)
         XFreeDeviceList (device_info);
 }
 
-void MouseManager::SetDisableWTyping (bool         state)
+void MouseManager::SetDisableWTyping (bool state)
 {
     if (property_from_name ("Synaptics Off"))
         SetDisableWTypingSynaptics (state);
@@ -1204,12 +1299,15 @@ set_tap_to_click_libinput (XDeviceInfo *device_info, bool   state)
 }
 
 static void
-set_tap_to_click (XDeviceInfo *device_info,  bool state,  bool left_handed,
+set_tap_to_click(XDeviceInfo *device_info,  bool state,  bool left_handed,
                   int one_finger_tap, int two_finger_tap, int three_finger_tap)
 {
-        if (property_from_name ("Synaptics Tap Action"))
-                set_tap_to_click_synaptics (device_info, state, left_handed,
-                                            one_finger_tap, two_finger_tap, three_finger_tap);
+    if (query_device_had_property(device_info,"Synaptics Tap Action")) {
+        set_tap_to_click_synaptics (device_info, state, 0,
+                                    one_finger_tap, two_finger_tap, three_finger_tap);
+        USD_LOG_SHOW_PARAM1(left_handed);
+        USD_LOG(LOG_DEBUG,"device name：%s",device_info->name);
+    }
 
         if (property_from_name ("libinput Tapping Enabled"))
                 set_tap_to_click_libinput (device_info, state);
@@ -1231,7 +1329,7 @@ void MouseManager::SetTapToClickAll ()
     int three_finger_tap = settings_touchpad->get(KEY_TOUCHPAD_THREE_FINGER_TAP).toInt();
 
     for (i = 0; i < numdevices; i++) {
-        set_tap_to_click (&devicelist[i], state, left_handed, one_finger_tap, two_finger_tap, three_finger_tap);
+        set_tap_to_click(&devicelist[i], state, left_handed, one_finger_tap, two_finger_tap, three_finger_tap);
     }
     XFreeDeviceList (devicelist);
 }
@@ -1256,7 +1354,8 @@ static void set_scrolling_libinput (XDeviceInfo *device_info,
     Atom prop, type;
     bool want_edge, want_2fg;
     bool want_horiz;
-    Display *display = QX11Info::display();
+    //Display *display = QX11Info::display();
+     Display * display = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
     prop = property_from_name ("libinput Scroll Method Enabled");
     if (!prop)
             return;
@@ -1274,7 +1373,8 @@ static void set_scrolling_libinput (XDeviceInfo *device_info,
      */
     if (want_2fg)
             want_edge = false;
-    qDebug ("setting scroll method on %s", device_info->name);
+
+    USD_LOG(LOG_DEBUG,"setting scroll method on %s", device_info->name);
     try {
         rc = XGetDeviceProperty (display, device, prop, 0, 2,
                                  False, XA_INTEGER, &type, &format, &nitems,
@@ -1290,7 +1390,7 @@ static void set_scrolling_libinput (XDeviceInfo *device_info,
                 XFree (data);
         XCloseDevice (display, device);
     } catch (int x) {
-        qWarning("Error in setting scroll method on \"%s\"", device_info->name);
+        USD_LOG(LOG_DEBUG,"Error in setting scroll method on \"%s\"", device_info->name);
     }
 
     /* Horizontal scrolling is handled by xf86-input-libinput and
@@ -1340,7 +1440,8 @@ void set_natural_scroll_synaptics (XDeviceInfo *device_info,
     unsigned char* data;
     long *ptr;
     Atom prop, type;
-    Display *display = QX11Info::display();
+     Display * display = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
+//    Display *display = QX11Info::display();
     prop = property_from_name ("Synaptics Scrolling Distance");
     if (!prop)
             return;
@@ -1384,7 +1485,7 @@ void set_natural_scroll_synaptics (XDeviceInfo *device_info,
 void set_natural_scroll_libinput (XDeviceInfo *device_info,
                                   bool       natural_scroll)
 {
-    qDebug ("Trying to set %s for \"%s\"",
+    USD_LOG (LOG_DEBUG,"Trying to set %s for \"%s\"",
             natural_scroll ? "natural (reverse) scroll" : "normal scroll",
             device_info->name);
     touchpad_set_bool (device_info, "libinput Natural Scrolling Enabled",
@@ -1419,32 +1520,33 @@ void MouseManager::SetNaturalScrollAll ()
 void set_touchpad_enabled (XDeviceInfo *device_info,
                            bool         state)
 {
-    XDevice *device;
-    Atom prop_enabled;
-    unsigned char data = state;
-    Display *display =  gdk_x11_get_default_xdisplay ();//QX11Info::display();//
-
-    prop_enabled = property_from_name ("Device Enabled");
-    if (!prop_enabled)
-        return;
-
+    Display* display = gdk_x11_get_default_xdisplay ();
+    XDevice * device;
     device = device_is_touchpad (device_info);
     if (device == NULL) {
         return;
     }
-    try {
-        XChangeDeviceProperty (display, device,
-                               prop_enabled, XA_INTEGER, 8,
-                               PropModeReplace, &data, 1);
-
-        XCloseDevice (display, device);
-        gdk_display_flush (gdk_display_get_default());
-    } catch (int x) {
-        qWarning("Error %s device \"%s\"",
-                  (state) ? "enabling" : "disabling",
-                   device_info->name);
+    int realformat;
+    unsigned long nitems, bytes_after;
+    unsigned char *data;
+    Atom realtype, prop;
+    prop = XInternAtom (display, "Device Enabled", False);
+    if (!prop) {
+        return;
     }
 
+    if (XGetDeviceProperty (display, device, prop, 0, 1, False,
+                            XA_INTEGER, &realtype, &realformat, &nitems,
+                            &bytes_after, &data) == Success) {
+
+        if (nitems == 1){
+            data[0] = state ? 1 : 0;
+            XChangeDeviceProperty(display, device, prop, XA_INTEGER, realformat, PropModeReplace, data, nitems);
+        }
+        XFree(data);
+    }
+
+    XCloseDevice (display, device);
 }
 
 void SetTouchpadEnabledAll (bool state)
@@ -1465,30 +1567,69 @@ bool SetDisbleTouchpad(XDeviceInfo *device_info,
     QString name;
     bool   state;
     name = device_info->name;
+    bool PReceiver = name.contains("Receiver", Qt::CaseInsensitive);
+    bool PWireless = name.contains("Wireless", Qt::CaseInsensitive);
     bool Pmouse = name.contains("Mouse", Qt::CaseInsensitive);
     bool Pusb = name.contains("USB", Qt::CaseInsensitive);
-    if(Pmouse && Pusb){
+    if(Pmouse && ( PWireless || PReceiver || Pusb )) {
         state = settings->get(KEY_TOUCHPAD_DISBLE_O_E_MOUSE).toBool();
-        if(state){
-            settings->set(KEY_TOUCHPAD_ENABLED, false);
-            return true;
-        }else {
-            settings->set(KEY_TOUCHPAD_ENABLED, true);
+        if(state){//如果开启插入鼠标禁用触摸板，则直接修改触摸板状态
+            SetTouchpadEnabledAll(!state);
             return true;
+        } else {//如果没有开启插入鼠标禁用触摸板，则根据触摸板总开关的状态，设置触摸板的enable状态
+            SetTouchpadEnabledAll(settings->get(KEY_TOUCHPAD_ENABLED).toBool());
         }
     }
     return false;
 }
 
+bool checkMouseExists()
+{
+    int numdevices, i;
+    XDeviceInfo *devicelist = XListInputDevices (QX11Info::display(), &numdevices);
+    if (devicelist == NULL){
+        return false;
+    }
+    for (i = 0; i < numdevices; i++) {
+        QString name;
+        name = devicelist[i].name;
+        bool PReceiver = name.contains("Receiver", Qt::CaseInsensitive);
+        bool PWireless = name.contains("Wireless", Qt::CaseInsensitive);
+        bool Pmouse = name.contains("Mouse", Qt::CaseInsensitive);
+        bool Pusb = name.contains("USB", Qt::CaseInsensitive);
+        if(Pmouse && ( PWireless || PReceiver || Pusb )) {
+            return true;
+        }
+    }
+    XFreeDeviceList (devicelist);
+    return false;
+}
+
+// while remove mouse
+void SetPlugRemoveMouseEnableTouchpad(QGSettings *settings)
+{
+    if(UsdBaseClass::isTablet()){
+        if(settings->get(KEY_TOUCHPAD_ENABLED).toBool()) {
+            SetTouchpadEnabledAll(settings->get(KEY_TOUCHPAD_ENABLED).toBool());
+        }
+    } else {
+        if(!checkMouseExists()) {
+            SetTouchpadEnabledAll(settings->get(KEY_TOUCHPAD_ENABLED).toBool());
+        }
+    }
+}
+
 void SetPlugMouseDisbleTouchpad(QGSettings *settings)
 {
     int numdevices, i;
     XDeviceInfo *devicelist = XListInputDevices (QX11Info::display(), &numdevices);
-    if (devicelist == NULL)
-            return;
+    if (devicelist == NULL) {
+        return;
+    }
     for (i = 0; i < numdevices; i++) {
-            if(SetDisbleTouchpad (&devicelist[i], settings))
-                break;
+        if(SetDisbleTouchpad (&devicelist[i], settings)) {
+            break;
+        }
     }
     XFreeDeviceList (devicelist);
 }
@@ -1500,7 +1641,8 @@ void SetTouchpadDoubleClick(XDeviceInfo *device_info, bool state)
     unsigned long nitems, bytes_after;
     unsigned char* data;
     Atom prop, type;
-    Display *display = gdk_x11_get_default_xdisplay ();//QX11Info::display();
+     Display * display = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
+//    Display *display = gdk_x11_get_default_xdisplay ();//QX11Info::display();
     prop = property_from_name ("Synaptics Gestures");
     if (!prop)
             return;
@@ -1551,7 +1693,8 @@ void MouseManager::SetBottomRightClickMenu(XDeviceInfo *device_info, bool state)
     unsigned char* data;
     long *ptr;
     Atom prop, type;
-    Display *display = gdk_x11_get_default_xdisplay ();//QX11Info::display();
+     Display * display = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
+//    Display *display = gdk_x11_get_default_xdisplay ();//QX11Info::display();
     prop = property_from_name ("Synaptics Soft Button Areas");
     if (!prop)
             return;
@@ -1605,7 +1748,6 @@ void MouseManager::SetBottomRightConrnerClickMenu(bool state)
 
 void MouseManager::TouchpadCallback (QString keys)
 {
-
     if (keys.compare(QString::fromLocal8Bit(KEY_TOUCHPAD_DISABLE_W_TYPING))==0) {
         SetDisableWTyping (settings_touchpad->get(keys).toBool());  //设置打字时禁用触摸板
 
@@ -1614,6 +1756,7 @@ void MouseManager::TouchpadCallback (QString keys)
         bool touchpad_left_handed = GetTouchpadHandedness (mouse_left_handed);
         SetLeftHandedAll (mouse_left_handed, touchpad_left_handed); //设置左右手
 
+
     } else if ((keys.compare(QString::fromLocal8Bit(KEY_TOUCHPAD_TAP_TO_CLICK))    == 0)
             || (keys.compare(QString::fromLocal8Bit(KEY_TOUCHPAD_ONE_FINGER_TAP))  == 0)
             || (keys.compare(QString::fromLocal8Bit(KEY_TOUCHPAD_TWO_FINGER_TAP))  == 0)
@@ -1628,15 +1771,19 @@ void MouseManager::TouchpadCallback (QString keys)
 
     } else if (keys.compare(QString::fromLocal8Bit(KEY_TOUCHPAD_NATURAL_SCROLL)) == 0) {
         SetNaturalScrollAll ();                             //设置上移下滚或上移上滚
+        USD_LOG(LOG_DEBUG,"set %s",KEY_TOUCHPAD_NATURAL_SCROLL);
 
     } else if (keys.compare(QString::fromLocal8Bit(KEY_TOUCHPAD_ENABLED)) == 0) {
         SetTouchpadEnabledAll (settings_touchpad->get(keys).toBool());//设置触摸板开关
-
-    } else if ((keys.compare(QString::fromLocal8Bit(KEY_MOTION_ACCELERATION)) == 0)
-            || (keys.compare(QString::fromLocal8Bit(KEY_MOTION_THRESHOLD)) == 0)) {
+    } else if ((keys.compare((KEY_MOTION_ACCELERATION)) == 0)
+            || (keys.compare((KEY_MOTION_THRESHOLD)) == 0)) {
+        SetMotionAll ();                                    //设置鼠标速度
+    }else if (0 == QString::compare(keys, QString(KEY_MOTION_ACCELERATION), Qt::CaseInsensitive)||
+              0 == QString::compare(keys, QString(KEY_MOTION_THRESHOLD), Qt::CaseInsensitive)){
         SetMotionAll ();                                    //设置鼠标速度
 
-    } else if (keys.compare(QString::fromLocal8Bit(KEY_TOUCHPAD_DISBLE_O_E_MOUSE)) == 0) {
+    }else if (keys == "motion-acceleration" || keys==KEY_MOTION_THRESHOLD){
+    }else if (keys.compare(QString::fromLocal8Bit(KEY_TOUCHPAD_DISBLE_O_E_MOUSE)) == 0) {
         SetPlugMouseDisbleTouchpad(settings_touchpad);      //设置插入鼠标时禁用触摸板
 
     } else if (keys.compare(QString::fromLocal8Bit(KEY_TOUCHPAD_DOUBLE_CLICK_DRAG)) == 0){
@@ -1647,6 +1794,8 @@ void MouseManager::TouchpadCallback (QString keys)
 
     } else if (keys.compare(QString::fromLocal8Bit(KEY_TOUCHPAD_MOUSE_SENSITVITY)) == 0){
 
+    } else {
+        USD_LOG(LOG_DEBUG,"keys:is skip..k%s", keys.toLatin1().data(),keys.toLatin1().data());
     }
 }
 
@@ -1659,14 +1808,19 @@ void MouseManager::SetMouseSettings ()
 
     SetMotionAll ();
     SetMiddleButtonAll (settings_mouse->get(KEY_MIDDLE_BUTTON_EMULATION).toBool());
+    SetMouseWheelSpeed (settings_mouse->get(KEY_MOUSE_WHEEL_SPEED).toInt());
+    SetPlugMouseDisbleTouchpad(settings_touchpad);
 
-    SetDisableWTyping (settings_touchpad->get(KEY_TOUCHPAD_DISABLE_W_TYPING).toBool());
+}
 
+void MouseManager::SetTouchSettings ()
+{
     SetTapToClickAll ();
     SetScrollingAll (settings_touchpad);
     SetNaturalScrollAll ();
     SetTouchpadEnabledAll (settings_touchpad->get(KEY_TOUCHPAD_ENABLED).toBool());
-    SetPlugMouseDisbleTouchpad(settings_touchpad);
+//    SetPlugMouseDisbleTouchpad(settings_touchpad);
+    SetPlugRemoveMouseEnableTouchpad(settings_touchpad);
     SetTouchpadDoubleClickAll(settings_touchpad->get(KEY_TOUCHPAD_DOUBLE_CLICK_DRAG).toBool());
     SetBottomRightConrnerClickMenu(settings_touchpad->get(KEY_TOUCHPAD_BOTTOM_R_C_CLICK_M).toBool());
 
@@ -1682,11 +1836,14 @@ GdkFilterReturn devicepresence_filter (GdkXEvent *xevent,
     MouseManager * manager = (MouseManager *) data;
 
     DevicePresence (gdk_x11_get_default_xdisplay (), xi_presence, class_presence);
-    if (xev->type == xi_presence)
-    {
+    if (xev->type == xi_presence) {
             XDevicePresenceNotifyEvent *dpn = (XDevicePresenceNotifyEvent *) xev;
-            if (dpn->devchange == DeviceEnabled)
-                    manager->SetMouseSettings ();
+            if (dpn->devchange == DeviceEnabled) {
+                USD_LOG(LOG_DEBUG,"new add deviced ID  : %d",dpn->deviceid);
+                manager->SetMouseSettings ();
+            } else if(dpn->devchange == DeviceRemoved) {
+                manager->SetTouchSettings();
+            }
     }
     return GDK_FILTER_CONTINUE;
 }
@@ -1714,13 +1871,22 @@ void MouseManager::MouseManagerIdleCb()
 
     time->stop();
 
-    QObject::connect(settings_mouse,SIGNAL(changed(QString)),
-                     this,SLOT(MouseCallback(QString)));
-    QObject::connect(settings_touchpad,SIGNAL(changed(QString)),
-                     this,SLOT(TouchpadCallback(QString)));
+//    connect(settings_mouse, &QGSettings::changed, this, &MouseManager::MouseCallback);
+//    connect(settings_touchpad, &QGSettings::changed, this, &MouseManager::TouchpadCallback);
+
+    QObject::connect(settings_mouse,SIGNAL(changed(QString)),  this,SLOT(MouseCallback(QString)));
+    QObject::connect(settings_touchpad,SIGNAL(changed(QString)),this,SLOT(TouchpadCallback(QString)));
+
     syndaemon_spawned = FALSE;
 
     SetDevicepresenceHandler ();
     SetMouseSettings ();
+    SetTouchSettings ();
+    SetDisableWTyping (settings_touchpad->get(KEY_TOUCHPAD_DISABLE_W_TYPING).toBool());
     SetLocatePointer (settings_mouse->get(KEY_MOUSE_LOCATE_POINTER).toBool());
+    if(checkMouseExists()){
+        SetPlugMouseDisbleTouchpad(settings_touchpad);
+    } else {
+        SetPlugRemoveMouseEnableTouchpad(settings_touchpad);
+    }
 }
