@@ -19,8 +19,13 @@
 #ifndef XRANDRPLUGIN_H
 #define XRANDRPLUGIN_H
 
+#include <QGuiApplication>
+#include <QX11Info>
+
+
 #include "plugin-interface.h"
 #include "xrandr-manager.h"
+#include "usd_base_class.h"
 
 class XrandrPlugin : public PluginInterface
 {
